<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services;

require __DIR__ . '/../Utilities/nusoap.php';

class Integracao extends \Zeedhi\Framework\Controller\Simple {

    protected $integracaoService;
    protected $util;
    protected $salesMessage;

	public function __construct(Services\Integracao $integracaoService, Utilities\Util $util, Utilities\SalesMessage $salesMessage) {
		$this->integracaoService = $integracaoService;
		$this->util = $util;
        $this->salesMessage     = $salesMessage;		
	}

	public function loadPeriodVend(Request\Filter $request, Response $response) {
        try {
            $produtos = $this->integracaoService->loadPeriodVend();
            
            foreach ($produtos as &$produto) {
            	$produto["VRUNITVEND"] = $this->util->formataDinheiro($produto["VRUNITVEND"]);
            	$produto["VRTOTALPROD"] = $this->util->formataDinheiro($produto["VRTOTALPROD"]);
            	$produto["VRACRITVEND"] = $this->util->formataDinheiro($produto["VRACRITVEND"]);
           		$produto["VRDESITVEND"] = $this->util->formataDinheiro($produto["VRDESITVEND"]);
           		$produto["IMPOSTO"] = $this->util->formataDinheiro($produto["IMPOSTO"]);
           		$produto["VRPEALPRODIT"] = $this->util->formataPorcentagem($produto["VRPEALPRODIT"]);
            }


            $response->addDataSet(new DataSet('GetVendasPeriodoRepo', $produtos));

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getIntegrationData (Request\Filter $request, Response $response) {
        try {
            $data = $this->integracaoService->getIntegrationData();

            $response->addDataSet(new DataSet('ParaERPRepository', $data));

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function updateERPParams (Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $params = $this->util->getParams($params);

            $NMENDWEBSER = $params['NMENDWEBSER'] == ''? null : $params['NMENDWEBSER'];

            $this->integracaoService->updateERPParams($NMENDWEBSER);
            
            $message = array('message' => $this->salesMessage->getMessage("alterado"));
            $response->addDataSet(new DataSet('UpdateParaERPRepository', $message));

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function getMovExport (Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            //items per page
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $FIRST = $request->getFilterCriteria()->getPageSize() * ($page-1);
            $LAST = ($FIRST + $pageSize);
            if($page > 1){
                $FIRST++;
            }
            $params = $this->util->getParams($params);

            //valida e põe na variavel
            $CDFILIAL = ($params['CDFILIAL']!='')? $params['CDFILIAL'] : "%%";
            $CDCAIXA  = ($params['CDCAIXA'] !='')? $params['CDCAIXA']  : "%%";
            //ja vem validado só formata
            $DTINIT   = $this->util->formatDate($params['DTINIT']);
            $DTFIM    = $this->util->formatDate($params['DTFIM']);

            $data = $this->integracaoService->getMovExport($CDFILIAL, $CDCAIXA, $DTINIT, $DTFIM, $FIRST, $LAST);

            $response->addDataSet(new DataSet('GetMovExportRepo', $data));

        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function exportRegister (Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $params = $this->util->getParams($params);
            $CDFILIAL = $params['CDFILIAL'];
            $CDCAIXA = $params['CDCAIXA'];
            $DTMOVTURCAIX = $params['DTMOVTURCAIX'];

        //do some stuff
    
            $wsdl = "http://localhost:9091/manage/backend/service/itfSD_Notifica_Out_Sync.wsdl";
            $user = 'FORSALE';
            $pass = 'Inicio@01';
            $folder = substr(__DIR__, 0, 1) . ":\\TEKNISA\\webservicegr\\logs\\";
            $date = date("dmY");
            $time = date("His");

            $CDLAYOUT = '001';

            $client = new \nusoap_client($wsdl, true);

            $client->setCredentials($user,$pass);

            $error = $client->getError();

            if($error){ 
                echo "Falha <pre>".print_r($error)."</pre>";
            }else {
                $caixasLiberados = $this->integracaoService->getCaixasLiberadosManualmente($CDFILIAL, $CDCAIXA, $DTMOVTURCAIX);
                //--- Writes the header of the request log
                $this->writeLogHeader(count($caixasLiberados), $date, $time, $folder);
                $logConsumo = '';
                $logSuccess = '';
                $logError = '';
                $countError = 0;
                $countSuccess = 0;
                foreach($caixasLiberados as $key=>$caixa){
                    $NRINSJURFILI  = $this->util->cleanSpecialChar($caixa['NRINSJURFILI']);
                    $NRINSJURFILI  = str_replace('-','',$NRINSJURFILI); 
                    $NRINSJURFILI  = preg_replace('/[^A-Za-z0-9\-]/', '', $NRINSJURFILI);
                    $CDFILIAL = $caixa['CDFILIAL'];
                    $CDLOJA = $caixa['CDLOJA'];
                    $CDCAIXA = $caixa['CDCAIXA'];
                    $DTMOVIMENTO = $caixa['DTMOVIMENTO'];
                    $QTDEVENDAS = $caixa['QTDEVENDAS'];
                    $DTMOVTURCAIX = $caixa["DTMOVTURCAIX"];
                    $ORIGEM = $caixa["ORIGEM"];
                    $dtconsumo = date('Y-m-d H:i:s');
                    $statusEnvio = $this->notificaCaixaFechado($client,$CDLAYOUT,$NRINSJURFILI,$CDFILIAL,$CDLOJA,$CDCAIXA,$DTMOVIMENTO,$QTDEVENDAS, $ORIGEM);
                    $logConsumo = 'DATA CONSUMO: '.$dtconsumo.' FILIAL: '.$CDFILIAL.' CDCAIXA: '.$CDCAIXA.' DATA MOVIMENTO: '.$DTMOVIMENTO.' ORIGEM: '.$ORIGEM.' STATUS: '.$statusEnvio." \n";
                    if(($statusEnvio == 0 || $statusEnvio == '0') && $statusEnvio !== null){
                        $this->integracaoService->updateCaixasLiberados($CDFILIAL, $CDCAIXA, $DTMOVTURCAIX);
                        $logSuccess .= $logConsumo;
                        $countSuccess++;
                    }else{
                        $logError .= $logConsumo;
                        $countError++; 
                    }

                }
                //--- Writes the success log
                $this->writeSuccessLog($countSuccess,$logSuccess,$date,$time,$folder);
                //--- Writes the error log
                $this->writeErrorLog($countError,$logError,$date,$time,$folder);       
            }

    
        //end stuff
            $message = ['message' => 'Exportado com Sucesso'];

            $response->addDataSet(new DataSet('ExportRegisterERPRepository', $message));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function exportRegister2 (Request\Filter $request, Response $response) {}
    public function writeLogHeader($countCaixasLiberados,$date,$time,$folder){
        if (!file_exists($folder)) {
            mkdir($folder, 0777, true);
        }
        $header = date("d-m-Y H:i:s") . " - Notificação de fechamento de caixas \n" ;
        $header .= $countCaixasLiberados." Caixas liberados \n";
        file_put_contents($folder."notificafechamento_" . $date . "_".$time."_request".".txt", $header, FILE_APPEND);
    }

    public function writeSuccessLog($countSuccess,$successLines,$date,$time,$folder){
        if (!file_exists($folder)) {
            mkdir($folder, 0777, true);
        }
        $logSuccess = $countSuccess." NOTIFICAÇÕES COM SUCESSO \n\n".$successLines;
        $logSuccess .= "\n__________________________________________________________________________\n\n\n";
        file_put_contents($folder."notificafechamento_" . $date . "_".$time."_success".".txt", $logSuccess, FILE_APPEND);
    }

    public function writeErrorLog($countError,$errorLines,$date,$time,$folder){
        if (!file_exists($folder)) {
            mkdir($folder, 0777, true);
        }
        $logError = $countError." NOTIFICAÇÕES COM ERRO \n\n".$errorLines;
        file_put_contents($folder."notificafechamento_" . $date . "_".$time."_error".".txt", $logError, FILE_APPEND);
    }

    public function writeExceptionLog($exception,$date,$time,$folder){
        if (!file_exists($folder)) {
            mkdir($folder, 0777, true);
        }
        $logException = "EXCEÇÕES ENCONTRADAS \n\n".$exception;
        file_put_contents($folder."notificafechamento_" . $date . "_".$time."_exception".".txt", $logException, FILE_APPEND);
    }



   public function notificaCaixaFechado($client,$CDLAYOUT,$NRINSJURFILI,$CDFILIAL,$CDLOJA,$CDCAIXA,$DTMOVIMENTO,$QTDEVENDAS, $ORIGEM){
        $notificaCaixaFechado = $client->call('notificaCaixaFechado',array(array(
                'CDLAYOUT' => $CDLAYOUT,
                'VENDAS' => array(
                        'NRINSJURFILI' => $NRINSJURFILI,
                        'CDFILIAL' => $CDFILIAL,
                        'CDLOJA' => $CDLOJA,
                        'CDCAIXA' => $CDCAIXA,
                        'DTMOVIMENTO' => $DTMOVIMENTO,
                        'QTDEVENDAS' => $QTDEVENDAS,
                        'ORIGEM' => $ORIGEM
                )
                )));
        $status = $notificaCaixaFechado["STATUS"];
        return $status;
    }
}