<?php
namespace Teknisa\MAN\Controllers;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\MAN\Services\RelDRFService;
use Teknisa\MAN\Services;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO\Response\Error;    

class RelDRF  extends  CrudWithParameterBag{ 
    
    const dataSourceDataApuracao = 'select_dataApuracao';
    
    protected $parameterBagColumnsApuracao = array('CDFILIAL', 'DTAPURROP', 'TDTAPURROP'),
              $utilService,
              $DayPartService;
              
    public function __construct ($crudService, $parameterBag, $entityManager, Utilities\Util $utilService, Manager $dataSourceManager,
                                 RelDRFService $RelDRFService, BirtReport $birtReport, ReportService $reportService) {
        $this->crudService          = $crudService;
        $this->parameterBag         = $parameterBag;
        $this->entityManager        = $entityManager;
        $this->utilService          = $utilService;
        $this->dataSourceManager    = $dataSourceManager;
        $this->DRFService           = $RelDRFService;
        $this->birtReport           = $birtReport;
        $this->reportService        = $reportService;
    }
    
    
     public function findRelatorio(DTO\Request\Row $request, DTO\Response $response) {
        $conditions = $this->utilService->getParamsAsArray($request);
        $params     = $this->DRFService->buildReportParameters($conditions);
        $this->getReport($params,$response);
    }
    
     public function findDataApuracao(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName   = 'select_dataApuracao';
        $dataSourceFilter       = RelDRF::buildDataSourceFilter($this->utilService->getParamsAsArray($request));
        $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $request->getFilterCriteria()->getDataSourceName(),
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        $this->parameterBagColumns = $this->parameterBagColumnsApuracao;
    	parent::find($newRequest, $response);
    }

    private function buildDataSourceFilter($conditions){
        $dataSourceFilter[0] = array(
            "columnName" => 'CDFILIAL',
            "operator"   => "=",
            "value"      => $conditions['CDFILIAL']
        );
        $dataSourceFilter[1] = array(
            "columnName" => 'DTAPURROP',
            "operator"   => "=",
            "value"      => NULL
        );
        $dataSourceFilter[2] = array(
            "columnName" => 'TDTAPURROP',
            "operator"   => "=",
            "value"      => 'T'
        );
        return $dataSourceFilter;
    }
    
    public function getReport($params, DTO\Response $response){
        try {
            $birtURL = $this->reportService->createRemoteReport('BIRT', $params['ReportName'], $params['ParametersRow'], true);
            $response->addDataset(new DataSet('URL',array('url'=>$birtURL)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage() , 500));
        }
    }
    
    
}