<?php
namespace Teknisa\MAN\Factories;

use Doctrine\ORM\EntityManager;

class ConsolidacaoAutomatica {

    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    public function buscaConsolidacoes($params){
        $sql = "SELECT CDFILIAL, CDLOJA, DTINICONSVENAUT, IDFREQCONSVEN, 
                HRCONSVENAUT, DSSTATUSULTCONS 
                  FROM PARAMCONSVENAUT 
                   WHERE (CDFILIAL = :CDFILIAL) 
                   AND (CDLOJA = :CDLOJA )
                   AND (NRORG = :NRORG )";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function updateConsolidacoes($params){
        $sql = "UPDATE PARAMCONSVENAUT
                    SET DTINICONSVENAUT = :DTINICONSVENAUT,
                    IDFREQCONSVEN       = :IDFREQCONSVEN,
                    HRCONSVENAUT        = :HRCONSVENAUT,
                    DSDIRARQPHP         = ' ',
                    DSDIRINSTALAPHP     = ' '
                WHERE CDFILIAL  = :CDFILIAL
                  AND CDLOJA   = :CDLOJA";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function insertConsolidacoes($params){
        $sql = "INSERT INTO PARAMCONSVENAUT
                  (DTINICONSVENAUT, IDFREQCONSVEN, HRCONSVENAUT,
                   CDFILIAL, CDLOJA, NRORG, DSDIRARQPHP, DSDIRINSTALAPHP) 
                VALUES
                  (:DTINICONSVENAUT, :IDFREQCONSVEN, :HRCONSVENAUT,
                   :CDFILIAL, :CDLOJA, :NRORG, ' ', ' ')";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function removeConsolidacoes($params){
        $sql = "DELETE FROM PARAMCONSVENAUT
                WHERE CDFILIAL         = :CDFILIAL
                AND   CDLOJA           = :CDLOJA
                AND   DTINICONSVENAUT  = :DTINICONSVENAUT
                AND   IDFREQCONSVEN    = :IDFREQCONSVEN
                AND   HRCONSVENAUT     = :HRCONSVENAUT
                AND   DSSTATUSULTCONS  = :DSSTATUSULTCONS";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function buscaProdutos($params) {
        $sql = "
            SELECT V.NRORG, V.CDFILIAL, V.CDCAIXA, CX.CDLOJA, V.NRSEQVENDA, V.CDCLIENTE,
                    TO_CHAR(V.DTENTRVENDA, 'DD/MM/YYYY') AS DTENTRVENDA,
                    V.IDTIPOIMPV, I.CDFILIALNF, I.NRLANCTONF, P.CDPRODUTO, P.NMPRODUTO, 
                    P.CDARVPROD, I.QTPRODVEND, I.NRSEQUITVEND, I.VRDESITVEND, P.CDPRODESTO, 
                    C.NMRAZSOCCLIE, V.NRCUPOMFIS, O.CDTIPOCONS, O.NMCONSUMIDOR, V.IDSITUVENDA, 
                    I.CDFILIALEST, NVL(I.NRLANCESTQ, L.NRLANCESTQ) AS NRLANCESTQ,
                    ((I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0))) - NVL(I.VRDESITVEND, 0)) AS TOTITEM,
                    (I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0)) AS VRUNITVEND,
                    NVL(V.IDTPFORPGVEN, 'V') AS IDTPFORPGVEN
                FROM VENDA V
                    JOIN ITEMVENDA I 
                        ON (V.CDFILIAL = I.CDFILIAL)
                            AND (V.CDCAIXA = I.CDCAIXA)
                            AND (V.NRSEQVENDA = I.NRSEQVENDA)
                            AND (V.NRORG = I.NRORG)
                    JOIN CLIENTE C 
                        ON (V.CDCLIENTE = C.CDCLIENTE)
                            AND (V.NRORG = C.NRORG)
                    JOIN PRODUTO P 
                        ON (I.CDPRODUTO = P.CDPRODUTO)
                            AND (V.NRORG = P.NRORG)
                    LEFT JOIN LANCTOESTOQ L 
                        ON (I.CDFILIAL = L.CDFILIAL)
                            AND (I.NRLANCTONF = L.NRLANCTONF)
                            AND (V.NRORG = L.NRORG)
                    JOIN CAIXA CX 
                        ON (V.CDFILIAL = CX.CDFILIAL)
                            AND (V.CDCAIXA = CX.CDCAIXA)
                            AND (V.NRORG = CX.NRORG)
                    LEFT JOIN CONSUMIDOR O 
                        ON (V.CDCLIENTE = O.CDCLIENTE)
                            AND (V.CDCONSUMIDOR = O.CDCONSUMIDOR)
                            AND (V.NRORG = O.NRORG)
                WHERE (V.CDFILIAL = :CDFILIAL)
                    AND (V.CDLOJA = :CDLOJA)
                    AND (V.NRORG = :NRORG)
                    AND (V.IDSITUVENDA IN ('O', 'D'))
                    AND (V.DTENTRVENDA BETWEEN :DTINICIAL AND :DTFINAL)
                    AND (I.CDFILIALEST IS NULL)
                    AND (I.NRLANCESTQ IS NULL)
                    AND (L.NRLANCESTQ IS NULL)
            ORDER BY V.DTENTRVENDA, V.CDCLIENTE, V.IDTPFORPGVEN, V.NRSEQVENDA, I.CDPRODUTO
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function buscaTipoRetiradaFilial($params) {
        $sql = "
            SELECT CDTIPORETI
                FROM PARAVEND
                WHERE CDFILIAL = :CDFILIAL 
                    AND NRORG = :NRORG
                    AND CDTIPORETI IS NOT NULL
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function updateStatusConsolidacao($params) {
        $sql = "
            UPDATE PARAMCONSVENAUT
                SET DSSTATUSULTCONS = :DSSTATUSULTCONS
                WHERE CDFILIAL = :CDFILIAL
                    AND CDLOJA = :CDLOJA
                    AND NRORG  = :NRORG
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function buscaNomeEmpresa($param) {
        $sql = "
            SELECT NMORGANIZACAO
                FROM ORGANIZACAO
                WHERE NRORG = :NRORG
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($param);
        
        return $std->fetchAll();
    }
}
